package viewer;

/**
 * Main calculations class. This class is the one doing the intense calculations.
 * It can be run in a seperate thread (implements Runnable interface). The
 * input for the calculation is given in the constructor, the calculations
 * results are reported through ICalcReport interface (also given in constructor)
 */
public class Calculator implements Runnable {
    /**
     * Callback interface that will recieve the calculation reports.
     */
    private ICalcReport calcReport;

    /**
     * The network data.
     */
    private NetData netData;

    /**
     * Valid permutations for each of the edges.
     * If when edge (a,b) is red, the permutation with index i is valid
     * then valids[a][b][i] == true, else false.
     */
    private boolean[][][] valids;

    /**
     * Current distribution function.
     */
    private double[] m_curDist;
    private int n, perms;

    /**
     * Current step number.
     */
    private int curStep;


    /**
     * Default constructor.
     * @param callBack Object that will recieve the calculation reports.
     * @param netData Input for the calculations - network data.
     */
    public Calculator(ICalcReport callBack, NetData netData) {
        this.calcReport = callBack;
        this.netData = netData;
    }

    /**
     * The main function of the class - does the calculations.
     */
    public void run() {
        StepData curData;


        // Preprocessing
        calcReport.StatusChange(ICalcReport.STATUS_STARTED);
        PrepareData();
        calcReport.StatusChange(ICalcReport.STATUS_PREPROC);

        // Steps
        for (int i = 0; i < netData.depth; i++) {
            curData = CalcStep();

            calcReport.StepDone(curData);

            if (curData.dist < netData.targetDistance)
                break;
            // Check for abort
            if (calcReport.isAborted()) {
                calcReport.StatusChange(ICalcReport.STATUS_ABORTED);
                return;
            }
        }
        // Done
        calcReport.StatusChange(ICalcReport.STATUS_FINISHED);
    }

    /**
     * Prepares data structures before the processing.
     */
    private void PrepareData() {
        curStep = 0;
        n = netData.nodesNum;
        perms = Permutation.factors[n];

        // Populate the valids structure
        valids = new boolean[netData.nodesNum][netData.nodesNum][];
        for (int a = 0; a < n; a++)
            for (int b = 0; b < n; b++) {
                valids[a][b] = new boolean[perms];
                if (a == b) { // For edges through which data passes
                    for (int i = 0; i < perms; i++)
                        if (Permutation.getNthVal(Permutation.IndexToPerm(n, i), a) == b)
                            valids[a][b][i] = true;
                } else { // For edges that do not pass data
                    for (int i = 0; i < perms; i++)
                        if (Permutation.getNthVal(Permutation.IndexToPerm(n, i), a) != b)
                            valids[a][b][i] = true;
                }
            }

        // Initial distribution function is [1, 0, ..., 0]
        // the first permutations is always ID
        m_curDist = new double[perms];
        for (int i = 0; i < perms; i++)
            m_curDist[i] = 0;
        m_curDist[Permutation.PermToIndex(n, 0L)] = 1;
    }

    /**
     * Calculates single step.
     * @return Calculated step data
     */
    private StepData CalcStep() {
        int i, j;
        StepData dataTarget = new StepData();

        // Possible permutations for current step
        boolean[] curValids = new boolean[perms];

        // Initially, all the permutations are possible
        for (i = 0; i < perms; i++)
            curValids[i] = true;

        // Go over all the edges
        for (int a = 0; a < n; a++)
            for (int b = 0; b < n; b++)
                if (netData.edges[curStep][a][b]) {
                    // If edges is red, we can reduce the number of possible permutations
                    for (i = 0; i < perms; i++)
                        curValids[i] = curValids[i] & valids[a][b][i];
                }

        // Count the total number of possible permutations
        int totalValid = 0;
        for (i = 0; i < perms; i++)
            if (curValids[i])
                totalValid++;

        // New distribution
        double[] newDist = new double[perms];
        for (i = 0; i < perms; i++)
            newDist[i] = 0;

        for (i = 0; i < perms; i++)
            if (curValids[i])
                for (j = 0; j < perms; j++)
                    if (m_curDist[j] != 0)
                        newDist[Permutation.composePerm(n, i, j)] += m_curDist[j] / totalValid;

        // Calculate distance
        double dist = 0;
        for (i = 0; i < perms; i++)
            dist += Math.abs(newDist[i] - (1.0 / perms));
        m_curDist = newDist;
        dataTarget.step = curStep;
        dataTarget.permutations = totalValid;
        dataTarget.dist = dist / 2.0f;
        dataTarget.distribution = newDist;
        curStep++;
        return dataTarget;
    }
}
