package viewer;

import java.awt.*;
import java.io.File;


import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.swing.*;

import org.w3c.dom.*;

/**
 * Main entrance class for the viewer applications.
 * The class is holding all the network properties and activates
 * the GuiPanel and Math models when needed.
 */
public class Controller implements FlowGraphListener {

    /**
     * Default early stop value
     */
    final static double EARLY_STOP_DEFAULT = 0.000001;

    private GuiPanel gui;

    /**
     * Red edges list
     * edge[j][i][k] == (i,j)->(k,j+1).
     */
    private boolean[][][] advacaryEdges;

    /**
     * Red nodes list.
     */
    private boolean[][] advacaryNodes;

    private double[] distances;
    private double[] distribution;
    private int[] permutations;

    /**
     * Early stop flag
     */
    private boolean earlyStop = true;

    /**
     * Early stop value
     */
    private double earlyStopVal = EARLY_STOP_DEFAULT;

    /**
     * Network properties
     */
    private NetworkProperties networkProperties = null;

    final static Color EDGE_COLOR = Color.black;
    final static Color GOOD_NODE_COLOR = Color.blue.darker();
    final static Color BAD_NODE_COLOR = Color.red;

    /**
     * Entry point to the controller.
     * @param args There are no program arguments
     */
    public static void main(String[] args) {
        Controller controller = new Controller();
    }

    /**
     * Create a window in which network properties can be chosen.
     */
    public Controller() {
        gui = new GuiPanel(this);
//        NetworkDialog networkDailog = new NetworkDialog(this);
    }

    /**
     * Randomly chooses a new set of red edges according
     * to the edges threshold.
     */
    public void RandomizeEdges() {
        int i,j,k;

        if (networkProperties == null)
            return;

        for (j = 0; j < networkProperties.numberOfLayers; j++)
            for (i = 0; i < networkProperties.activeComputers; i++)
                for (k = 0; k < networkProperties.activeComputers; k++) {
                    if (Math.random() < networkProperties.edgesThreshold)
                    // Red edge
                        advacaryEdges[j][i][k] = true;
                    else
                    // Good edge
                        advacaryEdges[j][i][k] = false;
                }
    }

    /**
     * Randomly chooses a new set of red nodes according
     * to the nodes threshold.
     */
    public void RandomizeNodes() {
        if (networkProperties == null)
            return;
        for (int j = 0; j < networkProperties.numberOfLayers + 1; j++)
            for (int i = 0; i < networkProperties.activeComputers; i++) {
                if (Math.random() < networkProperties.nodesThreshold)
                // Red node.
                    advacaryNodes[j][i] = true;
                else
                // Good node.
                    advacaryNodes[j][i] = false;
            }
    }

    /**
     * Starts the calculations on the current network, using the Math unit.
     */
    public void StartCalculation() {
        int i;

        if (networkProperties == null)
            return;

        NetData netdata = new NetData();
        netdata.edges = allRedEdges();
        netdata.nodesNum = networkProperties.activeComputers;
        netdata.depth = networkProperties.numberOfLayers;

        if(networkProperties.activeComputers>12)
        {
            JOptionPane.showMessageDialog(gui, "Cannot calculate for more than 10 nodes");
            return;
        }
        // If we want to use early stop.
        if (earlyStop)
            netdata.targetDistance = earlyStopVal;
        else
            netdata.targetDistance = 0;

        // Deleting the current Data.
        distances = new double[networkProperties.numberOfLayers+1];
        for (i = 0; i <= networkProperties.numberOfLayers; i++) {
            distances[i] = -1.0;
        }
        permutations = new int[networkProperties.numberOfLayers];
        for (i = 0; i < networkProperties.numberOfLayers; i++) {
            permutations[i] = -1;
        }
        distribution = null;

        // The first distance is known, we'll calculate it by ourselves
        distances[0] = 1 - 1.0/Permutation.factors[networkProperties.activeComputers];

        gui.repaint();

        ICalculations calcs = new MathUnit();
        calcs.StartCalculations(netdata);
        StepData sd;
        i = 0;
        //receiving all the data that was calculated in the math module
        while ((sd = calcs.GetStepData(i)) != null) {
            distribution = sd.distribution;
            distances[i+1] = sd.dist;
            permutations[i] = sd.permutations;
            i++;
            System.out.println(sd.step);
            System.out.println(sd.permutations);
            System.out.println(sd.dist);
        }
        gui.ShowDistributionGraph(distribution);
        gui.ShowDistanceGraph(distances);
        updateFlowGraph();
    }


    /**
     * Updates the edges and nodes that are displayed by FlowGraph.
     * To be called when values of edges or nodes have changed.
     */
    private void updateFlowGraph() {

        if (networkProperties == null)
            return;

        FlowGraph fg = gui.getFlowGraph();
        boolean redEdges[][][] = allRedEdges();

        for (int i = 0; i <= this.networkProperties.numberOfLayers; i++) {
            for (int j = 0; j < this.networkProperties.activeComputers; j++) {
                // Set color for the node
                if (advacaryNodes[i][j])
                    fg.setNodeColor(i, j, BAD_NODE_COLOR);
                else
                    fg.setNodeColor(i, j, GOOD_NODE_COLOR);

                if (i != this.networkProperties.numberOfLayers)
                    for (int k = 0; k < this.networkProperties.activeComputers; k++) {
                        // Set color for the edge
                        if (redEdges[i][j][k])
                            fg.setEdgeColor(i, j, k, EDGE_COLOR);
                        else // We don't want the good edges displayed at all, hence null for color
                            fg.setEdgeColor(i, j, k, null);
                    }
            }

            // Set distances label
            if (distances != null && distances[i] != -1)
                fg.setLayerLabel(i, Double.toString(distances[i]));
            else
                fg.setLayerLabel(i, null);

            if (i != this.networkProperties.numberOfLayers) {
                // Set permutations label
                if (permutations != null && permutations[i] != -1)
                    fg.setInterLayerLabel(i, Integer.toString(permutations[i]));
                else
                    fg.setInterLayerLabel(i, null);
            }
        }
        gui.repaint();
    }

    /**
     * Function that set new threshold values.
     * @param nodesThreshold Nodes threshold
     * @param edgesThreshold Edges threshold
     */
    public void setThresholds(float nodesThreshold, float edgesThreshold) {
        if (networkProperties == null)
            return;

        // Check input values
        if (nodesThreshold < 0 || nodesThreshold > 1)
            return;
        if (edgesThreshold < 0 || edgesThreshold > 1)
            return;

        networkProperties.nodesThreshold = nodesThreshold;
        networkProperties.edgesThreshold = edgesThreshold;

        // Randomize new values
        RandomizeNodes();
        RandomizeEdges();
        updateFlowGraph();
        return;
    }

    /**
     * Initializes the data by deleting all the data
     * that was calculated and clear all red edges and nodes.
     */
    private void InitializeData() {
        //assert that the network data is not null.
        if (networkProperties == null)
            return;
        int nodes = networkProperties.activeComputers;
        int hops = networkProperties.numberOfLayers;
        int i,j,k;

        advacaryEdges = new boolean[hops][nodes][nodes];
        advacaryNodes = new boolean[hops + 1][nodes];

        distances = new double[hops+1];
        for (i = 0; i < hops+1; i++) {
            distances[i] = -1.0;
        }

        permutations = new int[hops];
        for (i = 0; i < hops; i++) {
            permutations[i] = -1;
        }

        distribution = null;


        //Initialized the edges.
        for (j = 0; j < hops; j++)
            for (i = 0; i < nodes; i++)
                for (k = 0; k < nodes; k++)
                    advacaryEdges[j][i][k] = false;

        //Initialize the nodes.
        for (j = 0; j < hops + 1; j++)
            for (i = 0; i < nodes; i++)
                advacaryNodes[j][i] = false;
    }

    /**
     * Function the returns all the red edges in the network. Those are edges that
     * are tagged red, or that have red adjacent nodes
     */
    private boolean[][][] allRedEdges() {
        int i,j,k;

        boolean[][][] redEdges = new boolean[networkProperties.numberOfLayers][networkProperties.activeComputers][networkProperties.activeComputers];
        for (i = 0; i < networkProperties.numberOfLayers; i++)
            for (j = 0; j < networkProperties.activeComputers; j++)
                for (k = 0; k < networkProperties.activeComputers; k++) {
                    // Edge is red if...

                    // It is tagged red
                    if (advacaryEdges[i][j][k])
                        redEdges[i][j][k] = true;

                    // Or it comes from red node
                    else if (advacaryNodes[i][j])
                        redEdges[i][j][k] = true;

                    // Or if it goes into red node
                    else if (i != networkProperties.numberOfLayers && advacaryNodes[i + 1][k])
                        redEdges[i][j][k] = true;
                    else
                        redEdges[i][j][k] = false;
                }
        return redEdges;
    }

    public double getEarlyStopVal() {
        return earlyStopVal;
    }

    /**
     * Sets value of the early stop limit.
     * @param earlyStopVal Value from which the calculation will stop.
     */
    public void setEarlyStopVal(double earlyStopVal) {
        this.earlyStopVal = earlyStopVal;
    }

    /**
     * Activates or disactivates early stop feature.
     * @param earlyStop true for activation.
     */
    public void setEarlyStop(boolean earlyStop) {
        this.earlyStop = earlyStop;
    }

    /**
     * Getter of current network properties.
     * @return current network properties.
     */
    public NetworkProperties getNetworkProperties() {
        if(networkProperties == null)
            return new NetworkProperties(5, 20, 0.3f, 0.3f);
        return networkProperties;
    }

    /**
     * Loads entire network from a file.
     * @param file Previously saved XML files containing network structure.
     */
    public void loadFile(File file) {
        try {
            DocumentBuilderFactory factory =
                    DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse(file);
            setNetworkXml(document);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(gui, "Could not load network.\nError: " + e.getMessage());
        }
    }

    /**
     * Saves the current network structure into XML file.
     * @param file target file.
     */
    public void saveFile(File file) {
        try {
            if (networkProperties == null)
                return;
            Document document = getNetworkXml();
            if(document == null)
                throw new Exception("Could not get network structure XML");
            // Use a Transformer for output
            TransformerFactory tFactory =
                    TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();

            DOMSource source = new DOMSource(document);
            StreamResult result = new StreamResult(file);
            transformer.transform(source, result);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(gui, "Could not save network.\nError: " + e.getMessage());
        }


    }

    /**
     * Creates XML document containing network structure for persistance operations
     * @return XML stucture of the network
     */
    private Document getNetworkXml() {
        try {
            DocumentBuilderFactory factory =
                DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();

            Document document = builder.newDocument();
            Element documentElement = document.createElement("Network");
            document.appendChild(documentElement);
            Element properties = document.createElement("Properties");
            documentElement.appendChild(properties);
            properties.setAttribute("ActiveComputers", Integer.toString(networkProperties.activeComputers));
            properties.setAttribute("NumberOfLayers", Integer.toString(networkProperties.numberOfLayers));
            Element layer, node;
            String edges;
            int i, j, k;
            for (i = 0; i <= networkProperties.numberOfLayers; i++) {
                layer = document.createElement("Layer");
                documentElement.appendChild(layer);
                if (distances != null && distances[i] != -1.0)
                    layer.setAttribute("Distance", Double.toString(distances[i]));
                if (i != networkProperties.numberOfLayers) {
                    if (permutations != null && permutations[i] != -1)
                        layer.setAttribute("Permutations", Integer.toString(permutations[i]));
                }
                for (j = 0; j < networkProperties.activeComputers; j++) {
                    node = document.createElement("Node");
                    node.setAttribute("Red", Boolean.toString(advacaryNodes[i][j]));
                    layer.appendChild(node);
                    if (i != networkProperties.numberOfLayers) {
                        edges = new String();
                        for (k = 0; k < networkProperties.activeComputers; k++) {
                            edges += Integer.toString(advacaryEdges[i][j][k] ? 1 : 0) + " ";
                        }
                        node.appendChild(document.createTextNode(edges));
                    }
                }
            }
            return document;
        } catch (ParserConfigurationException e) {
            return null;
        }
    }

    /**
     * Loads new network structure from XML Document
     * @param document Structe of the network to load
     * @throws Exception when errors encountered during the loading
     */
    private void setNetworkXml(Document document) throws Exception {
        Element documentElement = document.getDocumentElement();
        if (documentElement == null)
            throw new Exception("No document element in the XML document");
        Element child = (Element) documentElement.getFirstChild();
        if (child == null || child.getNodeName() != "Properties")
            throw new Exception("No properties node in XML");
        int layers = Integer.parseInt(child.getAttribute("NumberOfLayers"));
        int comps = Integer.parseInt(child.getAttribute("ActiveComputers"));
        float nodesThreshold = getNetworkProperties().nodesThreshold;
        float edgesThreshold = getNetworkProperties().edgesThreshold;
        NetworkProperties props = new NetworkProperties(comps, layers, nodesThreshold, edgesThreshold);
        if (!props.isValid())
            throw new Exception("Network properties are not valid");
        setNetworkProperties(props);
        Attr attr;
        Element node;
        for (int i = 0; i <= layers; i++) {
            child = (Element) child.getNextSibling();
            if (child == null)
                break;
            attr = child.getAttributeNode("Distance");
            if (attr != null)
                distances[i] = Double.parseDouble(attr.getValue());
            attr = child.getAttributeNode("Permutations");
            if (attr != null)
                permutations[i] = Integer.parseInt(attr.getValue());
            node = (Element) child.getFirstChild();
            for (int j = 0; j < comps; j++) {
                if (node == null)
                    break;
                advacaryNodes[i][j] = Boolean.valueOf(node.getAttribute("Red")).booleanValue();
                Text edges = (Text) node.getFirstChild();
                if (edges != null) {
                    String edgesText = edges.getNodeValue();
                    String[] edgesTexts = edgesText.split(" ");
                    for (int k = 0; k < comps && k < edgesTexts.length; k++) {
                        if (edgesTexts[k] != null)
                            advacaryEdges[i][j][k] = (edgesTexts[k].matches("1"));
                    }
                }
                node = (Element) node.getNextSibling();
            }
        }
        updateFlowGraph();
    }

    /**
     * Set the network properties.
     * @param networkProperties Structure that holds the graph properties.
     */
    public void setNetworkProperties(NetworkProperties networkProperties) {

        if (!networkProperties.isValid())
            return;

        this.networkProperties = networkProperties;

        InitializeData();
        RandomizeNodes();
        RandomizeEdges();

        //If the gui already exits.
        if (gui == null)
            gui = new GuiPanel(this);

        FlowGraph fg = gui.getFlowGraph();
        fg.setDimensions(this.networkProperties.numberOfLayers + 1,
                this.networkProperties.activeComputers);
        fg.setFlowGraphListener(this);
        updateFlowGraph();
        gui.scroll.updateUI();
    }

    public void nodeClicked(int layerNum, int layerIndex, int button) {
        boolean newVal = (button == 0);
        if (advacaryNodes[layerNum][layerIndex] != newVal) {
            advacaryNodes[layerNum][layerIndex] = newVal;
            updateFlowGraph();
        }

    }

    public void edgeClicked(int sourceLayer, int sourceIndex, int destLayer, int destIndex, int button) {
        int y1, y2, x1, x2;
        if (sourceLayer < destLayer) {
            y1 = sourceLayer;
            y2 = destLayer;
            x1 = sourceIndex;
            x2 = destIndex;
        } else {
            y1 = destLayer;
            y2 = sourceLayer;
            x1 = destIndex;
            x2 = sourceIndex;
        }
        if (y2 - y1 != 1)
            return;

        boolean newVal = (button == 0);
        if (advacaryEdges[y1][x1][x2] != newVal) {
            advacaryEdges[y1][x1][x2] = newVal;
            updateFlowGraph();
        }

    }


}