package viewer;

import javax.swing.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.*;

/**
 * Graphical object representing a flow graph.
 *
 * Flow graph is a NxM graph, each row containing N nodes that can
 * be connected to the nodes in the next and previous nodes by an edge.
 *
 * Labels can be added for each layer and between layers.
 *
 * Mouse clicks on the network nodes and edges are monitored and can be reported using
 * the FlowGraphListener interface.
 */
public class FlowGraph extends JPanel implements MouseListener {

    // Default values
    final static Color LABELS_COLOR = Color.black;
    final static int NODE_SIZE = 10;
    final static int NODES_DIST = 70;
    final static int SHIFT = 10;

    // For user mouse click monitoring
    private NodeIndex originNodeIndex, distNodeIndex;

    // Dimentions of the graph
    private int layersNum;
    private int layerSize;

    // Node and edge colors
    private Color[][] nodeColors;
    private Color[][][] edgeColors;

    // Labels
    private String[] layerLabels;
    private String[] interLayerLabels;

    private FlowGraphListener flowGraphListener;

    /**
     * Default constructor - constracts the simplest network
     */
    public FlowGraph() {
        setDimensions(0, 0);
        addMouseListener(this);
    }

    /**
     * Change the dimensions of the flow graph.
     * @param layersNum Number of layers in network.
     * @param layerSize Number of nodes in each layer.
     */
    public void setDimensions(int layersNum, int layerSize) {
        this.layersNum = layersNum;
        this.layerSize = layerSize;

        if(layersNum > 0)
        {
            this.nodeColors = new Color[layersNum][layerSize];
            this.edgeColors = new Color[layersNum - 1][layerSize][layerSize];

            this.layerLabels = new String[layersNum];
            this.interLayerLabels = new String[layersNum - 1];
        }
    }

    /**
     * Changes certain node color.
     * @param layer layer of the node
     * @param index indes of the node in its layer
     * @param color new color.
     */
    public void setNodeColor(int layer, int index, Color color) {
        this.nodeColors[layer][index] = color;
        this.repaint();
    }

    /**
     * Changes edge color.
     * @param fromLayer source node layer.
     * @param sourceIndex source node index in layer.
     * @param destIndex target node index.
     * @param color the new color.
     */
    public void setEdgeColor(int fromLayer, int sourceIndex, int destIndex, Color color) {
        this.edgeColors[fromLayer][sourceIndex][destIndex] = color;
        this.repaint();
    }

    /**
     * Sets layer label.
     * @param layer layer to label.
     * @param label new label.
     */
    public void setLayerLabel(int layer, String label) {
        this.layerLabels[layer] = label;
        this.repaint();
    }

    /**
     * Sets layer between two layers.
     * @param layer first layer.
     * @param label new label.
     */
    public void setInterLayerLabel(int layer, String label) {
        this.interLayerLabels[layer] = label;
        this.repaint();
    }

    /**
     * Get the current FlowGraph listener on this graph.
     * @return the listener.
     */
    public FlowGraphListener getFlowGraphListener() {
        return flowGraphListener;
    }

    /**
     * Sets a new GraphListner for the graph.
     * @param flowGraphListener new listener.
     */
    public void setFlowGraphListener(FlowGraphListener flowGraphListener) {
        this.flowGraphListener = flowGraphListener;
    }


    public Dimension getPreferredSize() {
        int width = SHIFT + layerSize * NODES_DIST + NODE_SIZE;
        int height = SHIFT + layersNum * NODES_DIST + NODE_SIZE;
        return new Dimension(width, height);
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        if(layersNum == 0)
            return;
        for (int j = 0; j < layersNum; j++) {
            for (int i = 0; i < layerSize; i++) {
                g.setColor(nodeColors[j][i]);
                //Drawing the nodes.
                g.fillOval(i * NODES_DIST + SHIFT, j * NODES_DIST + SHIFT, NODE_SIZE, NODE_SIZE);

                //Drawing the adversary edges.
                if (j != layersNum - 1)
                    for (int k = 0; k < layerSize; k++) {
                        if (edgeColors[j][i][k] != null) {
                            g.setColor(edgeColors[j][i][k]);
                            g.drawLine(i * NODES_DIST + NODE_SIZE / 2 + SHIFT,
                                    j * NODES_DIST + NODE_SIZE / 2 + SHIFT,
                                    k * NODES_DIST + NODE_SIZE / 2 + SHIFT,
                                    (j + 1) * NODES_DIST + NODE_SIZE / 2 + SHIFT);
                        }
                    }
            }

            g.setColor(LABELS_COLOR);

            // Drawing the label of the layer
            if (layerLabels[j] != null)
                g.drawString(layerLabels[j]
                        , layerSize * NODES_DIST + SHIFT
                        , j * NODES_DIST + SHIFT + NODE_SIZE);

            // Drawing the interlayer labels
            if (j != layersNum - 1 && interLayerLabels[j] != null)
                g.drawString(interLayerLabels[j]
                        , layerSize * NODES_DIST + SHIFT
                        , (int) ((j + 0.5) * NODES_DIST + SHIFT + NODE_SIZE));
        }
    }

    /**
     * Function that is activated when the mouse was pressed.
     * It's purpose is to capture the position of the mouse when the
     * pressing happened. If we pressed on a valid node than
     * we tag this node as an origin node.
     * @param e (mouse event)
     */
    public void mousePressed(MouseEvent e) {
        int x = e.getX() - SHIFT;
        int y = e.getY() - SHIFT;
        originNodeIndex = GetNodeIndex(x, y);
    }

    /**
     * Function that is called when the mouse is released. If mouse is released over
     * valid node, that node is now tagged as the target node.
     * If the target node and the origin node are the same than we set
     * FlowGraphListener is informed of a node click. Otherwise, listener is informed
     * of a edge click
     * @param e
     */
    public void mouseReleased(MouseEvent e) {
        int x = e.getX() - SHIFT;
        int y = e.getY() - SHIFT;

        int button = e.isMetaDown() ? 1 : 0;

        distNodeIndex = GetNodeIndex(x, y);
        //If we pressed on anode in the begging and release on a node at the end.
        if (distNodeIndex != null && originNodeIndex != null) {
            //If we start end finish at the same node.
            if (distNodeIndex.y == originNodeIndex.y &&
                    distNodeIndex.x == originNodeIndex.x)
                fireNodeClickedEvent(originNodeIndex.y, originNodeIndex.x, button);
            //If the nodes are in adjacent layers.
            else
                fireEdgeClickedEvent(originNodeIndex.y, originNodeIndex.x,
                        distNodeIndex.y, distNodeIndex.x, button);
        }

        distNodeIndex = originNodeIndex = null;
    }

    private void fireEdgeClickedEvent(int sourceLayer, int sourceIndex, int destLayer, int destIndex, int button) {
        if (flowGraphListener != null)
            flowGraphListener.edgeClicked(sourceLayer, sourceIndex, destLayer, destIndex, button);
    }

    private void fireNodeClickedEvent(int layerNum, int layerIndex, int button) {
        if (flowGraphListener != null)
            flowGraphListener.nodeClicked(layerNum, layerIndex, button);
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    /**
     * Calculates what node lie in the proximity of given coordinates
     * @param x position on axis x
     * @param y position on axis y
     * @return Index of a node in the coordinates, null if no node there
     */
    public NodeIndex GetNodeIndex(int x, int y) {
        int divX,divY;
        double residueX,residueY;
        if (x < 0 || y < 0)
            return null;
        // Calculating the vertical position of the node
        divY = Math.round((y + NODES_DIST / 2) / NODES_DIST);
        residueY = (y - divY * NODES_DIST) - NODE_SIZE / 2;

        // Calculating the horizontal position of the node.
        divX = Math.round((x + NODES_DIST / 2) / NODES_DIST);
        residueX = (x - divX * NODES_DIST) - NODE_SIZE / 2;

        // Checking if we are in the proximity of the node.
        // If we are not close enough to any node we will not do nothing.
        if (Math.sqrt(Math.pow(residueX, 2) + Math.pow(residueY, 2)) > (NODE_SIZE * 1.5))
            return null;

        else {
            //Check if we are in the graph scope.
            if (divX < layerSize && divY < layersNum)
                return new NodeIndex(divX, divY);
        }
        return null;
    }

    /**
     * Class that represents the node position in the graph
     */
    static class NodeIndex {
        /**
         * index of the node in the layer.
         */
        public int x;

        /**
         * index of layer;
         */
        public int y;

        /**
         * @param i the position of the node in the layer.
         * @param j the layer number.
         */
        NodeIndex(int i, int j) {
            this.x = i;
            this.y = j;
        }
    }
}