package viewer;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;

/**
 * Class that implements the Gui model.
 * Responsible for the creation and updating of the GUI.
 * Includes the menu bar and a class flowGraph that create
 * representation of the network.
 */
public class GuiPanel extends JFrame implements ActionListener {

    JScrollPane scroll;

    Controller controller;
    JMenuBar bar = new JMenuBar();
    JMenu file = new JMenu("Network");
    JMenu simulation = new JMenu("Simulation");
    JMenuItem bNewGraph = new JMenuItem("New");
    JMenuItem bLoad = new JMenuItem("Load");
    JMenuItem bSave = new JMenuItem("Save");
    JMenuItem bRandomize = new JMenuItem("Randomize");
    JMenuItem bStart = new JMenuItem("Start");
    JCheckBoxMenuItem bEarlyStop = new JCheckBoxMenuItem("Early stop");
    JMenuItem bSetEarlyStop = new JMenuItem("Set early stop");

    final JFileChooser fileChooser = new JFileChooser();

    public FlowGraph getFlowGraph() {
        return flowGraph;
    }

    public void setFlowGraph(FlowGraph flowGraph) {
        this.scroll.setViewportView(flowGraph);
        this.flowGraph = flowGraph;
    }

    private FlowGraph flowGraph;

    /**
     * Constructor. The constructor create the menu bar and the
     * flowGraph.
     * @param controller pointer to the controller throw that pointer
     * we sending request to updating from the controller.
     */
    public GuiPanel(Controller controller) {
        this.controller = controller;
        flowGraph = new FlowGraph();
        file.add(bNewGraph);
        file.add(bLoad);
        file.add(bSave);
        file.add(bRandomize);
        bLoad.addActionListener(this);
        bSave.addActionListener(this);
        bRandomize.addActionListener(this);
        bNewGraph.addActionListener(this);

        bar.add(file);
        bStart.addActionListener(this);
        simulation.add(bStart);
        bEarlyStop.addActionListener(this);
        bEarlyStop.setState(true);
        bSetEarlyStop.addActionListener(this);
        simulation.add(bEarlyStop);
        simulation.add(bSetEarlyStop);
        bar.add(simulation);
        this.setJMenuBar(bar);

        scroll = new JScrollPane(flowGraph);
        scroll.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
        scroll.setDoubleBuffered(true);
        getContentPane().add(scroll, BorderLayout.CENTER);

        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        setSize(700, 600);
        setLocation(100, 100);
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Viewer Application");
        show();
    }

    /**
     * Function that create and show the distribution graph.
     * @param data array that include the probability of each permutation.
     */

    public void ShowDistributionGraph(double[] data) {
        double[] sortedData = new double[data.length];
        for (int i = 0; i < data.length; i++)
            sortedData[i] = data[i];
        java.util.Arrays.sort(sortedData);
        DistributionGraph graph = new DistributionGraph(sortedData);
        graph.show();
    }

    /**
     * Function that create and show the distance graph (distance between the current
     * distribution and the uniform distribution).
     * @param arrDist Array of distances for the graph
     */
    public void ShowDistanceGraph(double[] arrDist) {
        DistanceGraph graph = new DistanceGraph(arrDist);
        graph.show();
    }

    /**
     * This function is the standard actionPreformed. It's listening
     * to the menu bar, and sending the user request to the controller.
     * @param e represent the event that took place.
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == bRandomize) {
            ThresholdSlider slider = new ThresholdSlider(controller,
                    controller.getNetworkProperties().nodesThreshold,
                    controller.getNetworkProperties().edgesThreshold);
        }

        if (e.getSource() == bStart) {
            controller.StartCalculation();
        }
        if (e.getSource() == bLoad) {
            int returnVal = fileChooser.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fileChooser.getSelectedFile();
                controller.loadFile(file);
            }
        }
        if (e.getSource() == bSave) {
            int returnVal = fileChooser.showSaveDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fileChooser.getSelectedFile();
                controller.saveFile(file);
            }
        }

        //If we want to create a new graph.
        if (e.getSource() == bNewGraph) {
            NetworkDialog networkDailog = new NetworkDialog(controller.getNetworkProperties(), controller);
        }
        //Setting if we want to use or not use the early stop.
        if (e.getSource() == bEarlyStop) {
            if (bEarlyStop.getState())
                bSetEarlyStop.enable();
            else
                bSetEarlyStop.disable();
            controller.setEarlyStop(bEarlyStop.getState());
        }
        //Setting the early stop parameter.
        if (e.getSource() == bSetEarlyStop) {
            double earlyStopValue;
            earlyStopValue = controller.getEarlyStopVal();
            String st = JOptionPane.showInputDialog("Enter early stop value",
                    Double.toString(earlyStopValue));
            if (st != null) {
                try {
                    earlyStopValue = Double.parseDouble(st);
                    controller.setEarlyStopVal(earlyStopValue);
                } catch (NumberFormatException c) {
                    JOptionPane.showMessageDialog(null, "Please enter a number");
                    return;
                }
            }
        }
        repaint();
    }

}
