package viewer;

import javax.swing.*;

/**
 * Serves as an interface between the calculations unit and the other parts
 * the application
 */
public class MathUnit implements ICalculations, ICalcReport {
    public static final int STATUS_NOT_STARTED = -1;
    public static final int STATUS_WORKING = -2;
    public static final int STATUS_DONE = -3;

    /**
     * Current unit status
     */
    private int m_iStatus = STATUS_NOT_STARTED;

    /**
     * Calculator object
     */
    private Calculator m_calc;

    /**
     * Thread in which the calculations run
     */
    private Thread m_calcThread;

    /**
     * Container of step datas
     */
    private StepData[] m_stepsData;

    /**
     * Current step of the calculations
     */
    private int m_curCalcStep;

    private int m_totalSteps;
    private boolean m_iAborted;

    /**
     * Calculates (asynchroniously) probability functions for a run over
     * network with given parameters
     * @param netData Network data for the calculations
     * @return true if calculations started, false on error
     */
    public boolean StartCalculations(NetData netData) {
        if (m_iStatus == STATUS_WORKING)
            return false;
        JPanel panel = new JPanel();
        panel.setSize(300, 200);
        panel.setVisible(true);
        m_calc = new Calculator(new CalculationsReport(this, netData.depth), netData);
        m_calcThread = new Thread(m_calc);
        m_curCalcStep = 0;
        m_totalSteps = netData.depth;
        m_iAborted = false;
        m_stepsData = new StepData[m_totalSteps];
        m_iStatus = STATUS_WORKING;
        m_calcThread.start();
        return true;
    }

    /**
     * Returns calculated data for given step
     * @param stepNum Which step data to return
     * @return StepData class containing the data, null on error
     */
    public synchronized StepData GetStepData(int stepNum) {
        if (stepNum >= m_totalSteps)
            return null;
        // If the requested step is not yet calculated, wait() for it
        while (m_curCalcStep <= stepNum) {
            // In case the calculator is no longer working
            if (m_iStatus != STATUS_WORKING)
                return null;
            try {
                wait();
            } catch (InterruptedException e) {
            }
        }

        return m_stepsData[stepNum];
    }

    /**
     * Returns status of the MathUnit
     * @return STATUS_NOT_STARTED, STATUS_DONE or number of current step
     */
    public int GetCalcStatus() {
        if (m_iStatus != STATUS_WORKING)
            return m_iStatus;
        else
            return m_curCalcStep;
    }

    /**
     * Aborts currently running calculations. Returns after the calculations
     * are aborted (synchronious abort)
     * @return true on success, false on error
     */
    public synchronized boolean StopCalculations() {
        if (m_iStatus != STATUS_WORKING)
            return false;
        m_iAborted = true;
        // wait() until status is changed, only then return
        while (m_iStatus == STATUS_WORKING)
            try {
                wait();
            } catch (InterruptedException e) {
            }
        return true;
    }

    /**
     * Status change reports from the calculations unit
     * @param status New status
     */
    public synchronized void StatusChange(int status) {
        if (status == ICalcReport.STATUS_ABORTED || status == ICalcReport.STATUS_FINISHED)
            m_iStatus = STATUS_DONE;

        // Notify in case someone is waiting for status change
        notifyAll();
    }

    /**
     * Step data for newly calculated step
     * @param stepData Data for the step
     */
    public synchronized void StepDone(StepData stepData) {
//   assert (stepData.step < m_totalSteps);

//   assert (m_curCalcStep == stepData.step);
        m_stepsData[m_curCalcStep++] = stepData;

        // Notify in case someone is waiting for step data
        notifyAll();
    }

    /**
     * Check for abort
     * @return true if abort is needed, false otherwise
     */
    public boolean isAborted() {
        return m_iAborted;
    }
}