package viewer;

import java.awt.*;
import javax.swing.JFrame;
import java.awt.event.*;
import javax.swing.*;

/**
 * Class that responsible of the GuiPanel interface for the changes of the network
 * properties.
 */
public class NetworkDialog extends JFrame implements ActionListener {
    Controller controller;
    JTextField activeNodes = new JTextField(10);
    JLabel labelActiveNodes = new JLabel("Active Computers");
    JTextField layersNumber = new JTextField(10);
    JLabel labelLayerNumber = new JLabel("Layers number");
    JTextField nodesThreshold = new JTextField(10);
    JLabel labelNodesThreshold = new JLabel("Nodes Threshold");
    JTextField edgesThreshold = new JTextField(10);
    JLabel labelEdgesThreshold = new JLabel("Edges Threshold");
    private JButton bOK = new JButton("OK");
    private JButton bCancel = new JButton("Cancel");

    JTextField[] bTextButtons = {activeNodes, layersNumber, nodesThreshold, edgesThreshold};
    JLabel[] bLabelButtons = {labelActiveNodes, labelLayerNumber, labelNodesThreshold, labelEdgesThreshold};

    boolean graphExits = false;

    NetworkProperties netProp = new NetworkProperties(0, 0, 0, 0);

    /**
     * Constructor.
     * @param controller Pointer to the controller module.
     */
    public NetworkDialog(Controller controller) {
        int i;
        this.controller = controller;
        graphExits = false;

        //Layout the labels in a panel.
        JPanel labelsPane = new JPanel();
        labelsPane.setLayout(new GridLayout(0, 1));

        //Layout the text fields in a panel.
        JPanel fieldsPane = new JPanel();
        fieldsPane.setLayout(new GridLayout(0, 1));

        for (i = 0; i < bTextButtons.length && i < bLabelButtons.length; i++) {
            labelsPane.add(bLabelButtons[i]);
            bTextButtons[i].addActionListener(this);
            fieldsPane.add(bTextButtons[i]);
        }

        //Put the panels in another panel, labels on left,
        //text fields on right.
        JPanel contentPane = new JPanel();
        contentPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
        contentPane.setLayout(new BorderLayout());
        contentPane.add(labelsPane, BorderLayout.CENTER);
        contentPane.add(fieldsPane, BorderLayout.EAST);

        //The first focus is in the first field.
        bTextButtons[0].grabFocus();

        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
        buttonPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
        buttonPane.add(Box.createHorizontalGlue());
        buttonPane.add(bCancel);
        bCancel.addActionListener(this);
        buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
        buttonPane.add(bOK);
        buttonPane.add(Box.createHorizontalGlue());
        bOK.addActionListener(this);
        contentPane.add(buttonPane, BorderLayout.SOUTH);
        setContentPane(contentPane);

        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                if (!graphExits)
                    System.exit(0);
            }
        });

        setSize(300, 200);
        setLocation(200, 200);
        setTitle("Network dialog");
        show();
        this.nodesThreshold.setText("0");
        this.edgesThreshold.setText("0");
    }

    /**
     * Constructor. This constructor create a network dialog with a values that
     * are taken from the network properties parameter.
     * @param networkProp Structure that holding the current network properties
     * @param controller pointer to the controller.
     */
    public NetworkDialog(NetworkProperties networkProp, Controller controller) {
        this(controller);
        netProp = networkProp;
        graphExits = true;

        this.activeNodes.setText(Integer.toString(netProp.activeComputers));
        this.layersNumber.setText(Integer.toString(netProp.numberOfLayers));
        this.nodesThreshold.setText(Float.toString(netProp.nodesThreshold));
        this.edgesThreshold.setText(Float.toString(netProp.edgesThreshold));
    }

    /**
     * The action preformed method.
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == bOK) {
            try {
                int nodes = Integer.parseInt(activeNodes.getText());
                int hops = Integer.parseInt(layersNumber.getText());
                float nodesThres = Float.parseFloat(nodesThreshold.getText());
                float edgesThres = Float.parseFloat(edgesThreshold.getText());
                //checking the validation of the data.
                if (isValid(nodes, hops, nodesThres, edgesThres)) {//If we enter a valid data then we set the graph properties.
                    netProp.numberOfLayers = hops;
                    netProp.activeComputers = nodes;
                    netProp.nodesThreshold = nodesThres;
                    netProp.edgesThreshold = edgesThres;
                    //setting the graph properties.
                    controller.setNetworkProperties(netProp);
                    hide();
                }
            }
                    // In case that the parameters that the user entered are not numbers.
            catch (NumberFormatException c) {
                JOptionPane.showMessageDialog(null, "Please enter a number");
                return;
            }
        }
        if (e.getSource() == bCancel) {
            if (!graphExits)
                System.exit(0);
            else
                hide();
        }

        for (int i = 0; i < bTextButtons.length; i++) {
            if (e.getSource() == bTextButtons[i]) {//If we enter a data then we will shift focus to the next field.
                if (i + 1 < bTextButtons.length)
                    bTextButtons[i + 1].grabFocus();
                else
                    bOK.grabFocus();
            }
        }

    }

    /**
     * This function is checking if the data the user input is a valid input.
     */
    static boolean isValid(int nodes, int hops, float nodesThres, float edgesThreshold) {
        //Checking the nodes and hopes are not negative.
        if (nodes < 0 || hops < 0) {
            JOptionPane.showMessageDialog(null, "Please enter a number greater then 0");
            return false;
        }

        //checking that the thresholds values are between 0 to 1.
        if (nodesThres < 0 || nodesThres > 1 || edgesThreshold < 0 || edgesThreshold > 1) {
            JOptionPane.showMessageDialog(null, "Please enter a number between" +
                    "0 to 1 as threshold");
            return false;
        }
        return true;
    }

}