package viewer;

/**
 * Utility class for permutations
 */
public class Permutation {
    /**
     * Cache of factorial function up to fact(12) (maximum for int)
     */
    public static int[] factors = {0, 1, 2, 6, 24, 120, 720, 5040,
                                   40320, 362880, 3628800, 39916800,
                                   479001600};

    /**
     * Get permutation's index
     * @param n n
     * @param perm Permutation
     * @return Index of the permutation
     */
    public static int PermToIndex(int n, long perm) {
        // long containing bits that represent used digits from 0 to n-1
        int used_bits = 0;
        int used_mask;

        int index = 0;
        int val;
        for (int i = 1; i < n; i++) {
            val = (int) (perm & 0xf);
            used_mask = 1 << val;
            used_bits |= used_mask;
            used_mask >>= 1;
            while (used_mask != 0L) {
                if ((used_mask & used_bits) != 0L)
                    val--;
                used_mask >>= 1;
            }
            index = index * (n - i + 1) + val;
            perm >>= 4;
        }
        return index;
    }

    /**
     * Return permutation by it's index
     * @param n n
     * @param index Index of the requested permutation
     * @return The permutation that is represented be the index
     */
    public static long IndexToPerm(int n, int index) {
        // long containing bits that represent used digits from 0 to n-1
        int used_bits = 0;
        int used_mask;

        long perm = 0;
        int val, j;
        for (int i = 1; i <= n; i++) {
            val = 0;
            if (i == n)
                j = 0;
            else {
                j = index / factors[n - i];
                index %= factors[n - i];
            }
            used_mask = 1;
            while (j != 0) {
                if ((used_bits & used_mask) == 0)
                    j--;
                val++;
                used_mask <<= 1;
            }
            while ((used_bits & used_mask) != 0) {
                used_mask <<= 1;
                val++;
            }
            used_bits |= used_mask;
            perm |= (val << (i - 1) * 4);
        }

        return perm;
    }

    /**
     * Convert permutation to string
     * @param n n
     * @param perm Permuation
     * @return String representing the permuation
     */
    public static String PermToString(int n, long perm) {
        String str = "";
        for (int i = 0; i < n; i++) {
            str += "(" + (i + 1) + " " + (getNthVal(perm, i) + 1) + ") ";
        }
        return str;
    }

    /**
     * Returns perm(n)
     * @param perm Permutation
     * @param n Index
     */
    public static int getNthVal(long perm, int n) {
        return (int) ((perm >> 4 * n) & 0xf);
    }

    /**
     * Sets perm(n)=val - for building permutations
     * Note: Can create non-valid permutation
     * @param perm Permutation to change
     * @param n n
     * @param val New value for perm(n)
     * @return new permutation
     */
    public static long setNthVal(long perm, int n, long val) {
        return perm |= (val << 4 * n);
    }

    /**
     * Parses array and returns permutation that it represents
     * @param arr Array containing permutation
     * @return Permutation
     */
    public static long ArrToPerm(int[] arr) {
        long perm = 0;
        for (int i = 0; i < arr.length; i++)
            perm = setNthVal(perm, i, arr[i]);
        return perm;
    }

    /**
     * Composes two permutations
     * @param n n
     * @param p1 First permutation
     * @param p2 Second permutation
     * @return Composed permutation (p1*p2)
     */
    public static long composePerm(int n, long p1, long p2) {
        long res = 0;
        for (int i = 0; i < n; i++)
            res = setNthVal(res, i, getNthVal(p2, getNthVal(p1, i)));
        return res;
    }

    /**
     * Composes permutations given by indexes
     * @param n n
     * @param i1 Index of first permutation
     * @param i2 Index of second permutation
     * @return Index of the composed permutation
     */
    public static int composePerm(int n, int i1, int i2) {
        return PermToIndex(n, composePerm(n, IndexToPerm(n, i1), IndexToPerm(n, i2)));
    }

    /**
     * Check if permutation is valid
     * @param n n
     * @param perm Permutation to check
     * @return true if permutation is valid, false otherwise
     */
    public static boolean isValidPerm(int n, long perm) {
        long used_bits = 0;
        long used_mask;
        int val;
        for (int i = 0; i < n; i++) {
            val = getNthVal(perm, i);
            if (val >= n)
                return false;
            used_mask = 1 << val;
            if ((used_bits & used_mask) != 0)
                return false;
            used_bits |= used_mask;
        }
        return true;
    }

    /**
     * Test function for permutations
     * @param args
     */
    public static void main(String[] args) {
        int[] permArr1 = {3, 2, 1, 4, 0, 7, 5};
        int[] permArr2 = {1, 5, 6, 0, 4, 2, 1};

        long perm1 = ArrToPerm(permArr1);
        long perm2 = ArrToPerm(permArr2);
        System.out.println(isValidPerm(permArr1.length, perm1));
        System.out.println(isValidPerm(7, perm2));
        System.out.println(PermToString(permArr1.length, perm1));
        System.out.println(PermToString(7, perm2));
        System.out.println(PermToString(7, composePerm(7, perm1, perm2)));
        System.out.println(PermToString(7, composePerm(7, perm2, perm1)));
        System.out.println(PermToString(7, IndexToPerm(7, 0)));
    }
}